varying vec3 vertex,normal;

uniform sampler2D txt;

uniform float max_level,min_level;
uniform sampler2D txt_level_color;
uniform vec4 params;
uniform vec2 size;
varying vec2 text_coord;

uniform vec2 txt_box1;
uniform vec2 txt_box2;

vec4 GetLevelColor(float lv) 
{
	return texture2D(txt_level_color, vec2((lv - min_level)/(max_level-min_level),0.5));
}
float interpolate_bicubic(sampler2D tex, vec2 coord)
{
coord*=size;
	// transform the coordinate from [0,extent] to [-0.5, extent-0.5]
	vec2 coord_grid = coord-vec2(0.5);
	vec2 index = floor(coord_grid);
	vec2 fraction = coord_grid - index;
	vec2 one_frac = vec2(1.0) - fraction;
	vec2 one_frac2 = one_frac * one_frac;
	vec2 fraction2 = fraction * fraction;

	vec2 w0 = 1.0/6.0 * one_frac2 * one_frac;
	vec2 w1 = vec2(2.0/3.0) - 0.5 * fraction2 * (vec2(2.0)-fraction);
	vec2 w2 = vec2(2.0/3.0) - 0.5 * one_frac2 * (vec2(2.0)-one_frac);
	vec2 w3 = 1.0/6.0 * fraction2 * fraction;
	vec2 g0 = w0 + w1;
	vec2 g1 = w2 + w3;
	// h0 = w1/g0 - 1, move from [-0.5, extent-0.5] to [0, extent]
	vec2 h0 = (w1 / g0) - vec2(0.5) + index;
	vec2 h1 = (w3 / g1) + vec2(1.5) + index;
	// fetch the four linear interpolations
	
	h0/=size;
	h1/=size;
	float tex00 = texture2D(tex, h0).x;
	float tex10 = texture2D(tex, vec2(h1.x, h0.y)).x;
	float tex01 = texture2D(tex, vec2(h0.x, h1.y)).x;
	float tex11 = texture2D(tex, h1).x;
	// weigh along the y-direction
	tex00 = mix(tex01, tex00, g0.y);
	tex10 = mix(tex11, tex10, g0.y);
	// weigh along the x-direction
	return mix(tex10, tex00, g0.x);
}
vec4 get_spline_coefs(vec4 p)
{
	vec4 res;
	res.x = -0.5*p.x + 1.5*p.y - 1.5*p.z + 0.5*p.w;
	res.y = p.x - 2.5*p.y + 2.0*p.z - 0.5*p.w;
	res.z = -0.5*p.x + 0.5*p.z;
	res.w = p.y;
	return res;
}
float interpolate_bicubic111(sampler2D tex, vec2 coord)
{

	// transform the coordinate from [0,extent] to [-0.5, extent-0.5]
	vec2 coord_grid = coord*size-0.5;
	vec2 index = floor(coord_grid);
	vec2 fraction = coord_grid - index;
	vec2 one_frac = vec2(1.0) - fraction;
	//fraction=one_frac;

	mat4 vals;
	for(int i=0;i<4;i++)
	for(int j=0;j<4;j++)
		vals[j][i] = texture2D(tex, (index-0.5+vec2(i,j))/size).x;
	mat4 Am;

	Am[0][0] = 0.0 / 6.0;
	Am[1][0] = 6.0 / 6.0;
	Am[2][0] = 0.0 / 6.0;
	Am[3][0] = 0.0 / 6.0;

	Am[0][1] = -2.0 / 6.0;
	Am[1][1] = -3.0 / 6.0;
	Am[2][1] = 6.0 / 6.0;
	Am[3][1] = -1.0 / 6.0;

	Am[0][2] = 3.0 / 6.0;
	Am[1][2] = -6.0 / 6.0;
	Am[2][2] = 3.0 / 6.0;
	Am[3][2] = 0.0 / 6.0;

	Am[0][3] = -1.0 / 6.0;
	Am[1][3] = 3.0 / 6.0;
	Am[2][3] = -3.0 / 6.0;
	Am[3][3] = 1.0 / 6.0;

	//float res = vals[1][1];
	/*
	float res = vals[1][1]*one_frac.x*one_frac.y+
				vals[1][2]*fraction.x*one_frac.y+
				vals[2][1]*one_frac.x*fraction.y+
				vals[2][2]*fraction.x*fraction.y;*/
				
	vec4 yy = vec4(1.0,fraction.y,fraction.y*fraction.y,fraction.y*fraction.y*fraction.y);
	vec4 xx = vec4(1.0,fraction.x,fraction.x*fraction.x,fraction.x*fraction.x*fraction.x);
	
	//xx = Am*vals*transpose(Am)*xx;
	yy = Am*vals*transpose(Am)*yy;
	float res = dot(yy,xx);
	return res;
}

float interpolate_bicubic11(sampler2D tex, vec2 coord)
{

	// transform the coordinate from [0,extent] to [-0.5, extent-0.5]
	vec2 coord_grid = coord*size-0.5;
	
	vec2 index = floor(coord_grid);
	vec2 fraction = coord_grid - index;
	
	mat4 f;
	for(int i=0;i<4;i++)
	for(int j=0;j<4;j++)
		f[i][j] = texture2D(tex, (index-1.0+vec2(i,j))/size).x;
	

	float x = fraction.x;
	float y = fraction.y;

	
	vec2 v0 = vec2((x-1.0)*(x-2.0)*(x), (y-1.0)*(y-2.0)*(y))*(-1.0/6.0);
	vec2 v1 = vec2((x-1.0)*(x-2.0)*(x+1.0), (y-1.0)*(y-2.0)*(y+1.0))*(1.0/2.0);
	vec2 v2 = vec2((x)*(x-2.0)*(x+1.0), (y)*(y-2.0)*(y+1.0))*(-1.0/2.0);
	vec2 v3 = vec2((x-1.0)*(x)*(x+1.0), (y-1.0)*(y)*(y+1.0))*(1.0/6.0);
	float res = f[1][1] * v1.x*v1.y
				+f[1][2] * v1.x*v2.y
				+f[2][1] * v2.x*v1.y
				+f[2][2] * v2.x*v2.y

				+f[0][1] * v0.x*v1.y
				+f[0][2] * v0.x*v2.y
				+f[3][1] * v3.x*v1.y
				+f[3][2] * v3.x*v2.y

				+f[1][0] * v1.x*v0.y
				+f[1][3] * v1.x*v3.y
				+f[2][0] * v2.x*v0.y
				+f[2][3] * v2.x*v3.y

				+f[0][0] * v0.x*v0.y
				+f[0][3] * v0.x*v3.y
				+f[3][0] * v3.x*v0.y
				+f[3][3] * v3.x*v3.y;

	//res = f[1][1]*one_frac.x*one_frac.y+				f[1][2]*fraction.x*one_frac.y+				f[2][1]*one_frac.x*fraction.y+				f[2][2]*fraction.x*fraction.y;
	//return f[1][1];
	return res;
}

vec3 GetSpectr(float vl)
{
	vec3 cl[6];
	cl[0] = vec3(1.0,0.0,0.0);
	cl[1] = vec3(1.0,1.0,0.0);
	cl[2] = vec3(0.0,1.0,0.0);
	cl[3] = vec3(0.0,1.0,1.0);
	cl[4] = vec3(0.0,0.0,1.0);
	cl[5] = vec3(1.0,0.0,1.0);
	
	float id = floor(vl*4.9999);
	float al = vl*4.9999-id;
	return mix(cl[int(id)],cl[int(id)+1],al);
}

void main()
{
	vec2 text_coord1 = (text_coord.xy-txt_box1)/(txt_box2.xy-txt_box1.xy);

	//float val = params.x>0.5 ? interpolate_bicubic(txt, text_coord1.xy) : texture2D(txt, text_coord1.xy).x;
	//float val = filter(txt, text_coord1.xy,size);
	//float val = texture2D(txt, text_coord1.xy).x;
	
	float val = interpolate_bicubic111(txt, text_coord1.xy);
	if(max_level>0.005)val = texture2D(txt, text_coord1.xy).x;
	
	if(!(text_coord1==clamp(text_coord1,vec2(0.0),vec2(1.0))))val=0.0;

	vec3 color = vec3(clamp((val - min_level)/(max_level-min_level),0.0,1.0));
	
	color = GetSpectr(clamp(val *(128.0),0.0,1.0));
	if(val*(128.0)>0.49  && val*(128.0)<0.501)color=vec3(0.0,0.0,0.0);
	if(val*(128.0)>0.49-0.25  && val*(128.0)<0.501-0.25)color=vec3(0.0,0.0,1.0);
	if(val*(128.0)>0.49+0.25  && val*(128.0)<0.501+0.25)color=vec3(1.0,0.0,0.0);
	gl_FragColor = vec4(color,1.0);
}
